"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const appender_1 = require("./appender.cjs");
const importCommonJS_1 = require("./replace/importCommonJS.cjs");
const importMetaReplacer_1 = require("./replace/importMetaReplacer.cjs");
const importSpecifier_1 = require("./replace/importSpecifier.cjs");
const ttypescript_1 = require("./ttypescript.cjs");
class TypescriptTransformDualPackage extends typescript_transformer_common_1.TypescriptTransformPlugin {
    replaceMjsRequire;
    shadowPlugin;
    reEmitSourceFile(file) {
        if (file.isDeclarationFile)
            return;
        this.shadowPlugin?.emit(file);
    }
    configure(context, options) {
        delete this.replaceMjsRequire;
        this.replacement = new typescript_transformer_common_1.ReplacementSet(context, this.logger);
        this.registerNotifycation([typescript_1.default.SyntaxKind.SourceFile], this.reEmitSourceFile);
        const createAppendCallback = (extension) => {
            return (0, appender_1.appendCallback)(extension, this.resolver, this.logger);
        };
        if (this.engine === typescript_transformer_common_1.EngineKind.TTypescript) {
            if (options.module === typescript_1.default.ModuleKind.CommonJS) {
                this.replacement.push(new importMetaReplacer_1.ImportMetaReplacer(importMetaReplacer_1.ImportMetaReplacer.UrlToFilename));
                this.shadowPlugin = new ttypescript_1.ShadowTranform('.mjs', this);
            }
            else {
                this.replaceMjsRequire = new importCommonJS_1.ImportCommonJS(this.resolver);
                this.shadowPlugin = new ttypescript_1.ShadowTranform('.cjs', this);
            }
            this.replacement.push(new importSpecifier_1.ImportExportSpecifierReplacer(createAppendCallback('.js')));
            this.registerMutation(this.replacement.syntaxKinds, this.replacement.execute);
        }
        else {
            if (options.module === typescript_1.default.ModuleKind.CommonJS) {
                this.replacement.push(new importMetaReplacer_1.ImportMetaReplacer(importMetaReplacer_1.ImportMetaReplacer.UrlToFilename));
                this.replacement.push(new importSpecifier_1.ImportExportSpecifierReplacer(createAppendCallback(this.pluginOptions.cjs || '.cjs')));
            }
            else if ((0, typescript_transformer_common_1.isEsModule)(options.module)) {
                this.replaceMjsRequire = new importCommonJS_1.ImportCommonJS(this.resolver);
                this.replacement.push(new importSpecifier_1.ImportExportSpecifierReplacer(createAppendCallback(this.pluginOptions.mjs || '.js')));
            }
            this.registerMutation(this.replacement.syntaxKinds, this.replacement.execute);
        }
    }
    transformToplevelNodes(node) {
        // console.log(' - ', ts.SyntaxKind[node.kind]);
        if (!this.replaceMjsRequire) {
            return node;
        }
        if (this.replaceMjsRequire.check(node, this.logger)) {
            return this.replaceMjsRequire.replace(node, this.context, this.logger) || node;
        }
        return node;
    }
}
exports.default = new TypescriptTransformDualPackage().plugin;
//# sourceMappingURL=register.cjs.map
