import _build_script_typescript_transformer_common_0 from '@build-script/typescript-transformer-common';
const { EngineKind, isEsModule, ReplacementSet, TypescriptTransformPlugin } = _build_script_typescript_transformer_common_0;
import ts from 'typescript';
import { appendCallback } from "./appender.js";
import { ImportCommonJS } from "./replace/importCommonJS.js";
import { ImportMetaReplacer } from "./replace/importMetaReplacer.js";
import { ImportExportSpecifierReplacer } from "./replace/importSpecifier.js";
import { ShadowTranform } from "./ttypescript.js";
class TypescriptTransformDualPackage extends TypescriptTransformPlugin {
    replaceMjsRequire;
    shadowPlugin;
    reEmitSourceFile(file) {
        if (file.isDeclarationFile)
            return;
        this.shadowPlugin?.emit(file);
    }
    configure(context, options) {
        delete this.replaceMjsRequire;
        this.replacement = new ReplacementSet(context, this.logger);
        this.registerNotifycation([ts.SyntaxKind.SourceFile], this.reEmitSourceFile);
        const createAppendCallback = (extension) => {
            return appendCallback(extension, this.resolver, this.logger);
        };
        if (this.engine === EngineKind.TTypescript) {
            if (options.module === ts.ModuleKind.CommonJS) {
                this.replacement.push(new ImportMetaReplacer(ImportMetaReplacer.UrlToFilename));
                this.shadowPlugin = new ShadowTranform('.mjs', this);
            }
            else {
                this.replaceMjsRequire = new ImportCommonJS(this.resolver);
                this.shadowPlugin = new ShadowTranform('.cjs', this);
            }
            this.replacement.push(new ImportExportSpecifierReplacer(createAppendCallback('.js')));
            this.registerMutation(this.replacement.syntaxKinds, this.replacement.execute);
        }
        else {
            if (options.module === ts.ModuleKind.CommonJS) {
                this.replacement.push(new ImportMetaReplacer(ImportMetaReplacer.UrlToFilename));
                this.replacement.push(new ImportExportSpecifierReplacer(createAppendCallback(this.pluginOptions.cjs || '.cjs')));
            }
            else if (isEsModule(options.module)) {
                this.replaceMjsRequire = new ImportCommonJS(this.resolver);
                this.replacement.push(new ImportExportSpecifierReplacer(createAppendCallback(this.pluginOptions.mjs || '.js')));
            }
            this.registerMutation(this.replacement.syntaxKinds, this.replacement.execute);
        }
    }
    transformToplevelNodes(node) {
        // console.log(' - ', ts.SyntaxKind[node.kind]);
        if (!this.replaceMjsRequire) {
            return node;
        }
        if (this.replaceMjsRequire.check(node, this.logger)) {
            return this.replaceMjsRequire.replace(node, this.context, this.logger) || node;
        }
        return node;
    }
}
export default new TypescriptTransformDualPackage().plugin;
//# sourceMappingURL=register.js.map