"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShadowTranform = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const appender_1 = require("./appender.cjs");
const importMetaReplacer_1 = require("./replace/importMetaReplacer.cjs");
const importSpecifier_1 = require("./replace/importSpecifier.cjs");
class ShadowTranform extends typescript_transformer_common_1.TypescriptTransformPlugin {
    constructor(extension, parent) {
        super();
        this.pluginOptions = { extension, ...parent.pluginOptions };
        this.shadowProgram = parent.forkProgram({
            declaration: false,
            declarationMap: false,
            module: extension === '.cjs' ? typescript_1.default.ModuleKind.CommonJS : typescript_1.default.ModuleKind.ESNext,
        });
        this.writeFile = this.writeFile.bind(this);
    }
    configure(context, _options) {
        this.replacement1 = new typescript_transformer_common_1.ReplacementSet(context, this.logger);
        this.replacement2 = new typescript_transformer_common_1.ReplacementSet(context, this.logger);
        if (this.pluginOptions.extension === '.cjs') {
            this.replacement2.push(new importMetaReplacer_1.ImportMetaReplacer(importMetaReplacer_1.ImportMetaReplacer.UrlToFilename));
            this.registerMutation(this.replacement2.syntaxKinds, this.replacement2.execute);
        }
        else {
            // replace import from "commonjs"?
        }
        this.replacement1.push(new importSpecifier_1.ImportExportSpecifierReplacer((0, appender_1.appendCallback)(this.pluginOptions.extension, this.resolver, this.logger)));
    }
    transformToplevelNodes(node) {
        return this.replacement1.execute(node);
    }
    writeFile(fileName, text, writeByteOrderMark, onError, _sourceFiles, _data) {
        const dir = (0, path_1.dirname)(fileName);
        const base = (0, path_1.basename)(fileName);
        const newBase = base.replace(/\.jsx?(\.map$|$)/i, this.pluginOptions.extension + '$1');
        this.logger.debug('   * write: %s/{%s => %s}', dir, base, newBase);
        if (newBase.endsWith('.map')) {
            const mapData = JSON.parse(text);
            if (mapData.file) {
                mapData.file = mapData.file.replace(/\.js(x?)$/i, this.pluginOptions.extension + '$1');
            }
            else {
                (onError || this.logger.error)(`[dual-package] warning: sourcemap file ${dir}/${newBase} did not contains file.`);
            }
            text = JSON.stringify(mapData, null, 4);
        }
        else {
            if (/\.jsx?$/.test(base)) {
                text = text.trimEnd();
                const lastLineAt = text.lastIndexOf('\n');
                let lastLine = text.slice(lastLineAt + 1);
                const re = /\.js(x?\.map)$/;
                if (re.test(lastLine)) {
                    lastLine = lastLine.replace(re, this.pluginOptions.extension + '$1');
                    text = text.slice(0, lastLineAt) + '\n' + lastLine + '\n';
                }
                else {
                    console.log('inline-sourcemap currently not support');
                }
            }
        }
        return typescript_1.default.sys.writeFile(`${dir}/${newBase}`, text, writeByteOrderMark);
    }
    emit(file) {
        try {
            this.shadowProgram.emit(this.shadowProgram.getSourceFile(file.fileName), this.writeFile, undefined, false, {
                before: [this.plugin(this.shadowProgram, this.pluginOptions)],
            });
        }
        catch (e) {
            console.error('failed re-emit shadow program: %s', e.stack || e.message);
            throw e;
        }
    }
}
exports.ShadowTranform = ShadowTranform;
//# sourceMappingURL=ttypescript.cjs.map
